@@ -1,18 +1,37 @@
 import abc
-from typing import List
+from typing import List, Generator, Tuple
+
+from pydmt.utils.digester import Digester
 
 
-class Builder(metaclass=abc.ABCMeta):
+class Node(abc.ABC):
+    """
+    This is a source object which knows how to add his checksum to a checksum
+    calculation
+    """
+    @abc.abstractmethod
+    def add_to_digester(self, d: Digester) -> None:
+        pass
+
+    @abc.abstractmethod
+    def get_name(self) -> str:
+        pass
+
+
+class Builder(abc.ABC):
     """
     A Builder is what really builds things in the system.
     A builder knows on which inputs it relies and what
     outputs it generates (sometimes before build and sometimes
     after).
     """
-    __metaclass__ = abc.ABCMeta
 
     def get_name(self) -> str:
-        return self.__class__.__name__+" "+",".join(self.get_sources())+"->"+",".join(self.get_targets())
+        """
+        Override this to get better names
+        :return: the name of this builder
+        """
+        return f"{self.__class__.__name__}"
 
     @abc.abstractmethod
     def __init__(self):
@@ -32,7 +51,7 @@ class Builder(metaclass=abc.ABCMeta):
         """
 
     @abc.abstractmethod
-    def get_sources(self) -> List[str]:
+    def get_sources(self) -> List[Node]:
         """
         return the name of the source files for this builder
         If the builder takes a whole folder the list all the filers in that folder.
@@ -42,22 +61,77 @@ class Builder(metaclass=abc.ABCMeta):
         The @get_signature method is use for that.
         In the future the get_signature method will go away.
         """
-
-    def get_targets(self) -> List[str]:
-        """ return the names of the files that you are sure you produce. If you do not know them,
-        you must implement the 'get_targets_post_build' method and return None here."""
-        return []
-
-    def get_targets_post_build(self) -> List[str]:
-        """ find out the names of the files that you produced.
-        If you implemented the get_results_names and there you return all the results
-        that you created then you don't need to implement anything here """
-        return []
+    @abc.abstractmethod
+    def get_targets(self) -> List[Node]:
+        """
+        return list of targets
+        """
 
     @abc.abstractmethod
+    def yield_results(self) -> Generator[Tuple[str, str], None, None]:
+        """
+        Return the signatures and names of results
+        :return:
+        """
+
     def get_signature(self) -> str:
         """
         return the sha1 of anything that identifies the sources of the build
         Techically this is the sha1 of the file content of the list of files
         returned from :func:`~get_sources`
         """
+        d = Digester()
+        for source in self.get_sources():
+            source.add_to_digester(d)
+        return d.get_hexdigest()
+
+
+class File(Node):
+    """
+    This is a node which is a file
+    """
+    def __init__(self, filename: str):
+        self.filename = filename
+
+    def add_to_digester(self, d: Digester) -> None:
+        d.add_file(self.filename)
+
+    def get_name(self):
+        return self.filename
+
+
+class SourceFile(File):
+    pass
+
+
+class TargetFile(File):
+    pass
+
+
+class SourceFiles(Node):
+    """
+    This is a source of many files
+    """
+    def __init__(self, filenames: List[str], name: str):
+        self.filenames = filenames
+        self.name = name
+
+    def add_to_digester(self, d: Digester) -> None:
+        d.add_files(self.filenames)
+
+    def get_name(self):
+        return self.name
+
+
+class SourceFolder(Node):
+    """
+    This is a source of a single folder
+    """
+    def __init__(self, folder: str):
+        self.folder = folder
+
+    def add_to_digester(self, d: Digester) -> None:
+        d.add_folders(folders=[self.folder])
+
+    def get_name(self):
+        return self.folder
